<?php

class Kohana_User implements ArrayAccess {
    
    protected static $_instance;
    protected $_session;
    protected $_config;
    protected $_model;
    protected $_userInfo;
    protected static $_flag;

    /**
     * Вовзращает экземпляр класса (singleton-паттерн)
     *
     * @return self
     */     
    public static function instance($state = false) {
        self::$_flag = $state;
        if(!isset(self::$_instance))
        {
            $config = Kohana::$config->load('account');
            self::$_instance = new self($config);
        }
        return self::$_instance;
    }
    
    
    private function __construct($config = array()) {
        $this->_config = $config;
        $this->_session = Session::instance();

        $this->_model = new Model_Account;
        $this->_config['hash_key'] = $this->_model->getHashKey();
        $this->_config['hash_method'] = 'sha256';
        $isSignedIn = $this->isSignedIn();
        $this->_PrepareSemester();
        
        if($isSignedIn) {
            $id = $this->_session->get('ID');
            $this->_userInfo = $this->_getInfoFromDB($id);
            
            if (self::$_flag != true) {
                $this->_session->regenerate();
                $this->_session->set('start_time', time());
            }
        }
        
    }
    
    
    protected function _PrepareSemester() {
        $semesterID = $this->_session->get("SemesterID");
        if (!$semesterID) {
            $semesterID = $this->_model->GetCurSemesterID();
        }
        $this->SetSemester($semesterID);
    }
    
    public function SetSemester($semesterID) {
        $res = $this->_model->SetSemesterID($semesterID);
        if ($res >= 0)
            $this->_session->set("SemesterID", "$semesterID");
    }

    
    /**
     * Регистрирует нового пользователя и осуществляет вход.
     * Проверяет корректность кода активации и существование аккаунтов с такими же авторизационными данными.
     *
     * @param  string $code Код активации
     * @param  string $email E-Mail адресс
     * @param  string $login
     * @param  string $password
     * @return  array
     */          
    public function signUp($code, $email, $login, $password)
    {
        $model = &$this->_model;
        $account = Account::instance();
        
        $isValid = $model->isActivationCodeValid($code);
        if (!$isValid) {
            return array(false, 'invalid_code');
        }

        $isLogin = $account->isLoginExists($login);
        $isMail = $account->isMailExists($email);
        
        if ($isLogin) {
            return array(false, 'login_exists');
        } else if ($isMail) {
            return array(false, 'mail_exists');
        }
        
        $id = $model->activateAccount($login, $password, $email, $code);
        $this->completeSignIn($id, $this->hash($password));
        return array(true, 'ok');
    }

    /**
     * Проверяет корректность авторизационных данных и возвращает true, если авторизация прошла успешно, и false, если данные являются некорректными.
     *
     * @param  string $login
     * @param  string $password
     * @return  bool
     */      
    public function signIn($login, $password) {
        $id = (int)$this->_model->checkAuth($login, $password);
        if ($id === -1) {
            return false;
        } else {
            return $this->completeSignIn($id, $this->hash($password));
        }
    }
    
    protected function completeSignIn($id, $passhash) {
        $userHash = $this->hash($id.Request::$user_agent.Request::$client_ip).$this->_config['hash_key'];
        $passwordHash = $this->hash($passhash.$this->_config['hash_key']);
        Cookie::set('userhash', $passwordHash);
        $this->_userInfo = $this->_getInfoFromDB($id);
        $this->_session->regenerate();
        $this->_session->set('ID', $id);
        $this->_session->set('LoggedIn', true);
        $this->_session->set('UserHash', $this->hash($userHash));
        $this->_session->set('PasswordHash', $passwordHash);
        $this->_session->set('start_time', time());
        return TRUE;
    }    

    /**
     * Проверяет авторизационный статус пользователя и, если пользователь имеет useragent и IP, отличные от хранимых в сессии, осуществляет выход из текущего сеанса.
     *
     * @return  bool
     */   
    public function isSignedIn()
    {
        $session = &$this->_session;
        if($session->get('LoggedIn') && !$this->checkHash()) {
            $this->completeSignOut();
        }
        return $this->_session->get('LoggedIn');
    }
    
    protected function checkHash()
    {
        $id = $this->_session->get('ID');
        $userHash = $this->hash($id.Request::$user_agent.Request::$client_ip).$this->_config['hash_key'];
        $userCheck = $this->_session->get('UserHash') == $this->hash($userHash);
        $passCheck = Cookie::get('userhash') == $this->_session->get('PasswordHash');
        return $userCheck AND $passCheck;
    }    
   
    /**
     * Завершает текущий сеанс пользователя.
     *
     * @return  array
     */    
    public function signOut()
    {
        if($this->isSignedIn()) {
            return $this->completeSignOut();
        }
        return FALSE;
    }
    
    protected function completeSignOut()
    {
        $this->_session ->set('ID', FALSE)
                        ->set('LoggedIn', FALSE)
                        ->set('UserHash', FALSE);
        Cookie::delete('userhash');
        unset($this->_userInfo);
        $this->_session->restart();
        return TRUE;
    }

    /**
     * Проверяет корректность данного пароля для текущего пользователя.
     *
     * @param string $password Пароль
     * @return  bool
     */        
    public function checkPassword($password) {
        if(!$this->isSignedIn())
            return FALSE;
        $passhash = $this->hash($password);
        return $this->hash($passhash.$this->_config['hash_key']) === $this->_session->get('PasswordHash');
    }
    
    public function changePassword($old, $new)
    {
        if(!$this->checkPassword($old))
            return FALSE; 
        $this->_model->changePassword($this->offsetGet('ID'), $new);
        $passhash = $this->hash($this->hash($new).$this->_config['hash_key']);
        Cookie::set('userhash', $passhash);
        $this->_session->set('PasswordHash', $passhash);
        return TRUE;
    }
    
    public function changeLogin($login)
    {
        if(!$this->isSignedIn() || Account::instance()->isLoginExists($login))
            return FALSE;
        $this->_model->changeLogin($this->offsetGet('ID'), $login);
        return TRUE;
    }    
    
    public function changeMail($email)
    {
        if(!$this->isSignedIn() || Account::instance()->isMailExists($email))
            return FALSE;
        $token = md5(time().$this->offsetGet('EMail').$email);
        $this->_session->set('NewMail_Token', $token);
        $this->_session->set('NewMail_Adress', $email);
        return $token;
    }    
    
    public function completeChangeMail($token)
    {
        $email = $this->_session->get('NewMail_Adress');
        if($token == $this->_session->get('NewMail_Token') AND !Account::instance()->isMailExists($email))
        {
            $this->_model->changeMail($this->offsetGet('ID'), $email);
            return true;
        }
        else {
            return false;
        } 
    }        
    
    public function changeProfile($data)
    {
        if($this->offsetGet('Type') == 'teacher')
        {
            $this->_model->ChangeTeacherInfo(
                    $this->offsetGet('TeacherID'),
                    $data['lastName'], 
                    $data['firstName'], 
                    $data['secondName'], 
                    $data['jobPositionID'],
                    $data['departmentID']);
        }
    }
    
    // ---------------------------- [INFO] -------------------------------------
    
    /**
     * Возвращает массив, содержащий пользовательские данные.
     *
     * @return  array
     */
    public function getInfoAsArray()
    {
        if($this->isSignedIn())
            return $this->_userInfo + $this->_session->as_array();
        else
            return array();
    }
    
    protected function _getInfoFromDB($id)
    {
        $info = $this->_model->getPersonalInfo($id)->offsetGet(0);
        $info += $this->_model->getAccountInfo($id)->offsetGet(0);
        return $info;
    }

    // ---------------------------- [ARRAY] ------------------------------------
    
    public function offsetSet($offset, $value) {
        if(isset($this->_userInfo[$offset])) {
            return $this->_userInfo[$offset];
        } else {
            throw new Kohana_Exception('Invalid key: '.$offset);
        }
    }

    public function offsetGet($offset) {
        if(isset($this->_userInfo[$offset]))
        {
            return $this->_userInfo[$offset];
        }
        else
            return FALSE;
    }
    
    public function offsetUnset($offset) {
        if(isset($this->_userInfo[$offset]))
        {
            unset($this->_userInfo[$offset]);
        }
    }
    
    public function offsetExists($offset) {
        return isset($this->_userInfo[$offset]);
    }
    
    /**
     * Perform a hmac hash, using the configured method.
     *
     * @param   string  $str  string to hash
     * @return  string
     */
    protected function hash($str)
    {
        if ( ! $this->_config['hash_key'])
        {
            $this->_config['hash_key'] = $key = md5(time().Request::$client_ip);
            $this->_model->setHashKey($key);
        }
        return hash_hmac($this->_config['hash_method'], $str, $this->_config['hash_key']);
    }    
}